package com.fourc.mongotest

import scala.concurrent.{ ExecutionContext, Future }
import scala.util.{ Failure, Success }
import akka.actor.ActorSystem
import reactivemongo.api.collections.bson.BSONCollection
import reactivemongo.api.{ DefaultDB, FailoverStrategy, MongoConnectionOptions, MongoDriver }
import reactivemongo.api.indexes.{ DefaultIndexesManager, Index, IndexType }
import reactivemongo.bson.BSONDocument
import Logger.logger
import java.util.UUID

object MongoDatabase {
  private val IDField = "id"

  // TODO maybe this version is better:
  // private class MongoCollection( database: => Future[DefaultDB], name: String) {
  // ...
  // }
  private class MongoCollection(database: DefaultDB, name: String)(implicit ec: ExecutionContext) {

    private val index = Index(Seq((IDField, IndexType.Ascending)), Some(s"${name}index"), unique = true, background = true)

    def document1 = BSONDocument(
      "id" -> UUID.randomUUID().toString,
      "firstName" -> "Stephane",
      "lastName" -> "Godbillon",
      "age" -> 29)


    def checkIndex(): Future[Boolean] = {
      logger.debug(s"Checking index for $name")
      val manager = new DefaultIndexesManager(database)
      manager.onCollection(name).ensure(index)
    }

    def insertDocument(): Future[Unit] = {
      val collection: BSONCollection = database.collection(name)
      logger.debug("Inserting Document")
      val writeRes = collection.insert(document1)
      writeRes.onComplete {
        case Failure(e) =>
          logger.error("Exception in inserting document", e )
        case Success(writeResult) =>
          logger.debug(s"Successfully inserted document with result: $writeResult")
      }
      writeRes.map( _ =>())
    }
  }

}

class MongoDatabase(implicit system: ActorSystem) {

  import system.dispatcher
  import MongoDatabase._

  private val driver = new MongoDriver()
  private val connection = driver.connection(Seq("127.0.0.1")
    , options = MongoConnectionOptions(failoverStrategy = FailoverStrategy(retries = 10, delayFactor = _ => 1))
  )

  private def database = connection.database("testdb")

  private def collection1 = database.map { d => new MongoCollection(d, "testCollection1") }
  private def collection2 = database.map { d => new MongoCollection(d, "testCollection2") }
  private def collection3 = database.map { d => new MongoCollection(d, "testCollection3") }
  private def collection4 = database.map { d => new MongoCollection(d, "testCollection4") }

  def insertDocument() = {
    collection1.flatMap{ c => c.insertDocument() }
  }

  def checkIndex() = for {
    c1 <- collection1
//    c2 <- collection2
//    c3 <- collection3
//    c4 <- collection4
  } yield {
    c1.checkIndex()
//    c2.checkIndex()
//    c3.checkIndex()
//    c4.checkIndex()
  }

  def release() = {
    connection.close()
    driver.close()
  }

}
