package com.fourc.mongotest


import scala.util.Try
import akka.actor.ActorSystem
import Logger.logger
import scala.concurrent.Await
import scala.concurrent.duration._
import sys.process._

object Test extends App {

  val iteration = Try(args(0).toInt).recover {
    case _: ArrayIndexOutOfBoundsException =>
      logger.info("Number of iterations not supplied... default is 1")
      1
    case _: NumberFormatException =>
      logger.warn("The first argument is the number of iterations and must be a Int... default is 1")
      1
  }

  val system = ActorSystem("MongoTest")
  logger.info("Starting test")
  var exception = false
  iteration.map { it =>
    for {
      i <- 1 to it
    } yield {
      val m = new MongoDatabase()(system)
      try {
        logger.info(s"Executing iteration $i")
        val a = m.checkIndex()
        Await.result(a, 15.seconds)
        logger.debug("Checking index completed... going to terminate mongod")
        // TODO: modify the following line depending on your os
        "sudo killall mongod" !;
        Thread.sleep(2000)
        val b = m.insertDocument()
        Await.result(b, 60.seconds)
        logger.debug("Insert document completed... releasing resources")
      } catch {
        case e: Exception =>
          logger.error(s"Exception in iteration $i!!", e)
          exception = true
      } finally {
        m.release()
        logger.debug("Resources released")
        if (exception)
          System.exit(1)
      }
    }
  }
  logger.info("Execution completed... shutting down actor system")
  system.shutdown()
}
